package com.citra.pivot.demo;

import java.awt.BorderLayout;
import java.awt.Container;
import java.text.DateFormat;
import java.util.Date;

import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.border.TitledBorder;

import com.citra.pivot.PivotTable;
import com.citra.pivot.olap.DefaultOlapProvider;
import com.citra.pivot.olap.OlapDefinition;
import com.citra.pivot.olap.OlapUtils;
import com.citra.pivot.olap.data.DataSource;
import com.citra.pivot.olap.data.DataSourceException;
import com.citra.pivot.olap.metadata.OlapHierarchy;
import com.citra.pivot.table.DefaultTableLeafAggregator;
import com.citra.pivot.table.StandardTableMeasure;
import com.citra.pivot.table.TableCube;
import com.citra.pivot.table.TableDataSource;
import com.citra.pivot.table.TableDimension;
import com.citra.pivot.table.TableHierarchy;
import com.citra.pivot.table.TableMeasure;
import com.citra.pivot.table.TableSchema;
import com.citra.pivot.table.aggregator.Aggregators;
import com.citra.pivot.table.aggregator.Functions;
import com.citra.pivot.table.aggregator.RankAggregator;
import com.citra.pivot.table.context.MonthContext;
import com.citra.pivot.table.context.QuarterContext;
import com.citra.pivot.table.context.SingleGroupContext;
import com.citra.pivot.table.context.YearContext;
import com.citra.renderers.DateRenderer;
import com.citra.table.AdvancedJScrollPane;
import com.citra.table.AdvancedJTable;
import com.citra.table.DefaultListTableModel;
/**
 * Pivot table demo with in-memory data from a csv file.
 */
public class OlapTableDemo extends AbstractOlapDemo {
	/** the original two-dimensional table model */
	protected DefaultListTableModel dataModel;
	/** whether to show the original flat data */
	protected boolean showOriginalData;
/**
 *
 */
public OlapTableDemo() {
	this(false, true);
}
/**
 *
 */
public OlapTableDemo(boolean showOriginalData) {
	this(showOriginalData, true);
}
/**
 *
 */
public OlapTableDemo(boolean showOriginalData, boolean useRemote) {
	super(useRemote);
	this.showOriginalData = showOriginalData;
}
/**
 *
 */
protected DataSource createDataSource() {
	//get data
	return createDataSource(getDataModel());
}
/**
 *
 */
public static DataSource getTableDataSource() {
	DataSource source = createDataSource(new OlapTableReader().createTableModel("report.txt"));
	return source;
}
/**
 *
 */
protected static DataSource createDataSource(DefaultListTableModel dataModel) {

	//start with creating schema
	TableSchema schema = new TableSchema("My Schema");

	//create product dimension
	TableDimension productsDim = schema.createDimension("Products");
	//create the default hierarchy for the product dimension
	TableHierarchy productsHier = productsDim.createHierarchy("Products");
	//add levels
	productsHier.createLevel("All Products", new SingleGroupContext("All Products"));
	productsHier.createLevel("Categories", dataModel.findColumn("Category"));
	productsHier.createLevel("Products", dataModel.findColumn("Product"));

	//create time dimension
	TableDimension timeDim = schema.createDimension("Time");
	//create the default hierarchy for the time dimension
	TableHierarchy timeHier = timeDim.createHierarchy("Time");
	//add levels
	timeHier.createLevel("All Years", new SingleGroupContext("All Years"));
	timeHier.createLevel("Year", new YearContext(dataModel.findColumn("Order Date")));
	timeHier.createLevel("Quarter", new QuarterContext(dataModel.findColumn("Order Date")));
	timeHier.createLevel("Month", new MonthContext(dataModel.findColumn("Order Date")));

	//create customer dimension
	TableDimension customerDim = schema.createDimension("Customer");
	//create the default hierarchy for the customer dimension
	TableHierarchy customerHier = customerDim.createHierarchy("Customer");
	//add levels
	customerHier.createLevel("All Customers", new SingleGroupContext("All Customers"));
	customerHier.createLevel("Customer Country", dataModel.findColumn("Customer Country"));
	customerHier.createLevel("Customer City", dataModel.findColumn("Customer City"));
	customerHier.createLevel("Customer", dataModel.findColumn("Customer"));

	//create a cube
	TableCube cube = schema.createCube("sales");
	//add dimensions to the cube
	cube.addDimension(productsDim);
	cube.addDimension(timeDim);
	cube.addDimension(customerDim);

	//create some standard measures
	TableMeasure sales = cube.createMeasure("Net Sales", Double.class, dataModel.findColumn("Net Sales"), Aggregators.getSum());
	TableMeasure maxSales = cube.createMeasure("Max Sales", Double.class, dataModel.findColumn("Net Sales"), Aggregators.getMax());
	StandardTableMeasure orders = cube.createMeasure("# of Orders", Integer.class, dataModel.findColumn("Net Sales"), Aggregators.getSum());
	orders.setLeafAggregator(new DefaultTableLeafAggregator(dataModel.findColumn("Net Sales"), Functions.getCountFunction()));

	//and a derived one
	TableMeasure rankOfSales = cube.createDerivedMeasure("Sales Rank", Double.class, sales, new RankAggregator(timeDim));

	//create TableDataSource
	TableDataSource dataSource = new TableDataSource(schema, dataModel);

	//compile schema - this will validate and create the schema
	try {
		System.out.println("compiling cube");
		dataSource.compile();
		System.out.println("finished compiling cube");
	} catch (Exception e) {
		e.printStackTrace();
		return null;
	}

	return dataSource;
}
/**
 *
 */
protected void customizeTable(PivotTable pivotTable, DefaultOlapProvider provider) {
	//get pivot definition in order to modify it
	OlapDefinition def = provider.getOlapDefinition();
	//modify definition
	//pivot the customer hierarchy on the rows
	OlapHierarchy customerHier = OlapUtils.findHierarchyByName(provider.getCube(), "Customer");
	def.setHierarchy(customerHier, OlapDefinition.ROW_TYPE);
	//pivot the time hierarchy on the rows as well, after customer
	OlapHierarchy timeHier = OlapUtils.findHierarchyByName(provider.getCube(), "Time");
	def.setHierarchy(timeHier, OlapDefinition.ROW_TYPE, 1);
	//pivot the products hierarchy on the rows as well, after customer
	OlapHierarchy productsHier = OlapUtils.findHierarchyByName(provider.getCube(), "Products");
	def.setHierarchy(productsHier, OlapDefinition.COLUMN_TYPE);
	//place the measure hierarchy on the filter area
	def.setHierarchy(def.getMeasureHierarchy(), OlapDefinition.FILTER_TYPE);

	//set new definition
	try {
		provider.setOlapDefinition(def);
	} catch (DataSourceException e) {
		e.printStackTrace();
	}
}
/**
 *
 */
private DefaultListTableModel getDataModel() {
	if (dataModel == null) {
		dataModel = new OlapTableReader().createTableModel("report.txt");
		System.out.println("read " + dataModel.getRowCount() + " lines of data.");
	}
	return dataModel;
}
/**
 *
 */
protected Container getFramePanel(Container tablePanel) {
	if (!showOriginalData) {
		dataModel = null; //for garbage collection to occur
		return super.getFramePanel(tablePanel);
	}

	//create table
	AdvancedJTable table = new AdvancedJTable(getDataModel());
	table.getTableAssistant().setMaxColumns(5);
	//set renderers
	DateRenderer dr = (DateRenderer) table.getDefaultRenderer(Date.class);
	dr.setDateFormat(DateFormat.getDateInstance());
	//create scroll pane
	AdvancedJScrollPane scroller = new AdvancedJScrollPane(table);

	//create panel for table
	JPanel panel = new JPanel(new BorderLayout());
	panel.add(scroller);
	panel.setBorder(new TitledBorder("Data model"));

	//create and return split pane
	JSplitPane splitter = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, tablePanel, panel);

	return splitter;
}
/**
 *
 */
public static void main(String[] args) {
	//run demo
	OlapTableDemo demo = new OlapTableDemo();
	try {
		//show
		demo.show();
	}
	catch (Exception e) {
		e.printStackTrace();
	}
}
}
